"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .clienterror import ClientError
from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .accountresponse import AccountResponse, AccountResponseTypedDict, AccountType
    from .addsenderop import AddSenderResponse, AddSenderResponseTypedDict
    from .cancelledmessage import CancelledMessage, CancelledMessageTypedDict
    from .cancelmessagesop import (
        CancelMessagesRequest,
        CancelMessagesRequestTypedDict,
        CancelMessagesResponse,
        CancelMessagesResponseTypedDict,
    )
    from .deletesenderop import (
        DeleteSenderRequest,
        DeleteSenderRequestTypedDict,
        DeleteSenderResponse,
        DeleteSenderResponseTypedDict,
    )
    from .errorresponse import ErrorResponse, ErrorResponseTypedDict
    from .errorresponse_error import ErrorResponseError, ErrorResponseErrorData
    from .getincomingmessagesop import (
        GetIncomingMessagesRequest,
        GetIncomingMessagesRequestTypedDict,
        GetIncomingMessagesResponse,
        GetIncomingMessagesResponseTypedDict,
    )
    from .getmessagesop import (
        GetMessagesRequest,
        GetMessagesRequestTypedDict,
        GetMessagesResponse,
        GetMessagesResponseTypedDict,
    )
    from .getmmspriceop import (
        GetMmsPriceRequestBody,
        GetMmsPriceRequestBodyTypedDict,
        GetMmsPriceResponse,
        GetMmsPriceResponseTypedDict,
    )
    from .getsmspriceop import (
        GetSmsPriceRequestBody,
        GetSmsPriceRequestBodyTypedDict,
        GetSmsPriceResponse,
        GetSmsPriceResponseTypedDict,
    )
    from .getsubaccountdetailsop import (
        GetSubaccountDetailsRequest,
        GetSubaccountDetailsRequestTypedDict,
    )
    from .incomingmessage import IncomingMessage, IncomingMessageTypedDict
    from .listincomingmessagesop import (
        ListIncomingMessagesRequest,
        ListIncomingMessagesRequestTypedDict,
        ListIncomingMessagesResponse,
        ListIncomingMessagesResponseTypedDict,
    )
    from .listmessagesop import (
        ListMessagesRequest,
        ListMessagesRequestTypedDict,
        ListMessagesResponse,
        ListMessagesResponseTypedDict,
    )
    from .message import Message, MessageTypedDict
    from .messagetype import MessageType
    from .mmsmessage import (
        Attachments,
        AttachmentsTypedDict,
        MmsMessage,
        MmsMessageTypedDict,
        Recipients,
        RecipientsTypedDict,
    )
    from .no_response_error import NoResponseError
    from .phonenumberwithcid import PhoneNumberWithCid, PhoneNumberWithCidTypedDict
    from .pingresponse import PingResponse, PingResponseTypedDict
    from .price import Price, PriceTypedDict
    from .removedincomingmessage import (
        RemovedIncomingMessage,
        RemovedIncomingMessageTypedDict,
    )
    from .removeincomingmessagesop import (
        RemoveIncomingMessagesRequest,
        RemoveIncomingMessagesRequestTypedDict,
        RemoveIncomingMessagesResponse,
        RemoveIncomingMessagesResponseTypedDict,
    )
    from .responsevalidationerror import ResponseValidationError
    from .sdkerror import SDKError
    from .security import Security, SecurityTypedDict
    from .sender import Sender, SenderTypedDict
    from .sender_input import SenderInput, SenderInputTypedDict
    from .sendmmsop import (
        SendMmsRequestBody,
        SendMmsRequestBodyTypedDict,
        SendMmsResponse,
        SendMmsResponseTypedDict,
    )
    from .sendsmsop import (
        SendSmsRequestBody,
        SendSmsRequestBodyTypedDict,
        SendSmsResponse,
        SendSmsResponseTypedDict,
    )
    from .setdefaultsenderop import (
        SetDefaultSenderRequest,
        SetDefaultSenderRequestTypedDict,
        SetDefaultSenderResponse,
        SetDefaultSenderResponseTypedDict,
    )
    from .smsmessage import (
        SmsMessage,
        SmsMessageRecipients,
        SmsMessageRecipientsTypedDict,
        SmsMessageTypedDict,
    )
    from .smstype import SmsType

__all__ = [
    "AccountResponse",
    "AccountResponseTypedDict",
    "AccountType",
    "AddSenderResponse",
    "AddSenderResponseTypedDict",
    "Attachments",
    "AttachmentsTypedDict",
    "CancelMessagesRequest",
    "CancelMessagesRequestTypedDict",
    "CancelMessagesResponse",
    "CancelMessagesResponseTypedDict",
    "CancelledMessage",
    "CancelledMessageTypedDict",
    "ClientError",
    "DeleteSenderRequest",
    "DeleteSenderRequestTypedDict",
    "DeleteSenderResponse",
    "DeleteSenderResponseTypedDict",
    "ErrorResponse",
    "ErrorResponseError",
    "ErrorResponseErrorData",
    "ErrorResponseTypedDict",
    "GetIncomingMessagesRequest",
    "GetIncomingMessagesRequestTypedDict",
    "GetIncomingMessagesResponse",
    "GetIncomingMessagesResponseTypedDict",
    "GetMessagesRequest",
    "GetMessagesRequestTypedDict",
    "GetMessagesResponse",
    "GetMessagesResponseTypedDict",
    "GetMmsPriceRequestBody",
    "GetMmsPriceRequestBodyTypedDict",
    "GetMmsPriceResponse",
    "GetMmsPriceResponseTypedDict",
    "GetSmsPriceRequestBody",
    "GetSmsPriceRequestBodyTypedDict",
    "GetSmsPriceResponse",
    "GetSmsPriceResponseTypedDict",
    "GetSubaccountDetailsRequest",
    "GetSubaccountDetailsRequestTypedDict",
    "IncomingMessage",
    "IncomingMessageTypedDict",
    "ListIncomingMessagesRequest",
    "ListIncomingMessagesRequestTypedDict",
    "ListIncomingMessagesResponse",
    "ListIncomingMessagesResponseTypedDict",
    "ListMessagesRequest",
    "ListMessagesRequestTypedDict",
    "ListMessagesResponse",
    "ListMessagesResponseTypedDict",
    "Message",
    "MessageType",
    "MessageTypedDict",
    "MmsMessage",
    "MmsMessageTypedDict",
    "NoResponseError",
    "PhoneNumberWithCid",
    "PhoneNumberWithCidTypedDict",
    "PingResponse",
    "PingResponseTypedDict",
    "Price",
    "PriceTypedDict",
    "Recipients",
    "RecipientsTypedDict",
    "RemoveIncomingMessagesRequest",
    "RemoveIncomingMessagesRequestTypedDict",
    "RemoveIncomingMessagesResponse",
    "RemoveIncomingMessagesResponseTypedDict",
    "RemovedIncomingMessage",
    "RemovedIncomingMessageTypedDict",
    "ResponseValidationError",
    "SDKError",
    "Security",
    "SecurityTypedDict",
    "SendMmsRequestBody",
    "SendMmsRequestBodyTypedDict",
    "SendMmsResponse",
    "SendMmsResponseTypedDict",
    "SendSmsRequestBody",
    "SendSmsRequestBodyTypedDict",
    "SendSmsResponse",
    "SendSmsResponseTypedDict",
    "Sender",
    "SenderInput",
    "SenderInputTypedDict",
    "SenderTypedDict",
    "SetDefaultSenderRequest",
    "SetDefaultSenderRequestTypedDict",
    "SetDefaultSenderResponse",
    "SetDefaultSenderResponseTypedDict",
    "SmsMessage",
    "SmsMessageRecipients",
    "SmsMessageRecipientsTypedDict",
    "SmsMessageTypedDict",
    "SmsType",
]

_dynamic_imports: dict[str, str] = {
    "AccountResponse": ".accountresponse",
    "AccountResponseTypedDict": ".accountresponse",
    "AccountType": ".accountresponse",
    "AddSenderResponse": ".addsenderop",
    "AddSenderResponseTypedDict": ".addsenderop",
    "CancelledMessage": ".cancelledmessage",
    "CancelledMessageTypedDict": ".cancelledmessage",
    "CancelMessagesRequest": ".cancelmessagesop",
    "CancelMessagesRequestTypedDict": ".cancelmessagesop",
    "CancelMessagesResponse": ".cancelmessagesop",
    "CancelMessagesResponseTypedDict": ".cancelmessagesop",
    "DeleteSenderRequest": ".deletesenderop",
    "DeleteSenderRequestTypedDict": ".deletesenderop",
    "DeleteSenderResponse": ".deletesenderop",
    "DeleteSenderResponseTypedDict": ".deletesenderop",
    "ErrorResponse": ".errorresponse",
    "ErrorResponseTypedDict": ".errorresponse",
    "ErrorResponseError": ".errorresponse_error",
    "ErrorResponseErrorData": ".errorresponse_error",
    "GetIncomingMessagesRequest": ".getincomingmessagesop",
    "GetIncomingMessagesRequestTypedDict": ".getincomingmessagesop",
    "GetIncomingMessagesResponse": ".getincomingmessagesop",
    "GetIncomingMessagesResponseTypedDict": ".getincomingmessagesop",
    "GetMessagesRequest": ".getmessagesop",
    "GetMessagesRequestTypedDict": ".getmessagesop",
    "GetMessagesResponse": ".getmessagesop",
    "GetMessagesResponseTypedDict": ".getmessagesop",
    "GetMmsPriceRequestBody": ".getmmspriceop",
    "GetMmsPriceRequestBodyTypedDict": ".getmmspriceop",
    "GetMmsPriceResponse": ".getmmspriceop",
    "GetMmsPriceResponseTypedDict": ".getmmspriceop",
    "GetSmsPriceRequestBody": ".getsmspriceop",
    "GetSmsPriceRequestBodyTypedDict": ".getsmspriceop",
    "GetSmsPriceResponse": ".getsmspriceop",
    "GetSmsPriceResponseTypedDict": ".getsmspriceop",
    "GetSubaccountDetailsRequest": ".getsubaccountdetailsop",
    "GetSubaccountDetailsRequestTypedDict": ".getsubaccountdetailsop",
    "IncomingMessage": ".incomingmessage",
    "IncomingMessageTypedDict": ".incomingmessage",
    "ListIncomingMessagesRequest": ".listincomingmessagesop",
    "ListIncomingMessagesRequestTypedDict": ".listincomingmessagesop",
    "ListIncomingMessagesResponse": ".listincomingmessagesop",
    "ListIncomingMessagesResponseTypedDict": ".listincomingmessagesop",
    "ListMessagesRequest": ".listmessagesop",
    "ListMessagesRequestTypedDict": ".listmessagesop",
    "ListMessagesResponse": ".listmessagesop",
    "ListMessagesResponseTypedDict": ".listmessagesop",
    "Message": ".message",
    "MessageTypedDict": ".message",
    "MessageType": ".messagetype",
    "Attachments": ".mmsmessage",
    "AttachmentsTypedDict": ".mmsmessage",
    "MmsMessage": ".mmsmessage",
    "MmsMessageTypedDict": ".mmsmessage",
    "Recipients": ".mmsmessage",
    "RecipientsTypedDict": ".mmsmessage",
    "NoResponseError": ".no_response_error",
    "PhoneNumberWithCid": ".phonenumberwithcid",
    "PhoneNumberWithCidTypedDict": ".phonenumberwithcid",
    "PingResponse": ".pingresponse",
    "PingResponseTypedDict": ".pingresponse",
    "Price": ".price",
    "PriceTypedDict": ".price",
    "RemovedIncomingMessage": ".removedincomingmessage",
    "RemovedIncomingMessageTypedDict": ".removedincomingmessage",
    "RemoveIncomingMessagesRequest": ".removeincomingmessagesop",
    "RemoveIncomingMessagesRequestTypedDict": ".removeincomingmessagesop",
    "RemoveIncomingMessagesResponse": ".removeincomingmessagesop",
    "RemoveIncomingMessagesResponseTypedDict": ".removeincomingmessagesop",
    "ResponseValidationError": ".responsevalidationerror",
    "SDKError": ".sdkerror",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "Sender": ".sender",
    "SenderTypedDict": ".sender",
    "SenderInput": ".sender_input",
    "SenderInputTypedDict": ".sender_input",
    "SendMmsRequestBody": ".sendmmsop",
    "SendMmsRequestBodyTypedDict": ".sendmmsop",
    "SendMmsResponse": ".sendmmsop",
    "SendMmsResponseTypedDict": ".sendmmsop",
    "SendSmsRequestBody": ".sendsmsop",
    "SendSmsRequestBodyTypedDict": ".sendsmsop",
    "SendSmsResponse": ".sendsmsop",
    "SendSmsResponseTypedDict": ".sendsmsop",
    "SetDefaultSenderRequest": ".setdefaultsenderop",
    "SetDefaultSenderRequestTypedDict": ".setdefaultsenderop",
    "SetDefaultSenderResponse": ".setdefaultsenderop",
    "SetDefaultSenderResponseTypedDict": ".setdefaultsenderop",
    "SmsMessage": ".smsmessage",
    "SmsMessageRecipients": ".smsmessage",
    "SmsMessageRecipientsTypedDict": ".smsmessage",
    "SmsMessageTypedDict": ".smsmessage",
    "SmsType": ".smstype",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gsmservice_gateway import models, utils
from gsmservice_gateway._hooks import HookContext
from gsmservice_gateway.types import OptionalNullable, UNSET
from gsmservice_gateway.utils import get_security_from_env
from gsmservice_gateway.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional


class Incoming(BaseSDK):
    def list(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListIncomingMessagesResponse:
        r"""List the received SMS messages

        Get the details of all received messages from your account incoming messages box. This method supports pagination so you have to pass `page` (number of page with received messages which you want to access) and a `limit` (max of received messages per page) named parameters. Messages are fetched from the latest one. This method will accept maximum **50** as `limit` parameter value.

        As a successful result a `ListIncomingMessagesResponse` object will be returned with `result` property of type `List[IncomingMessage]` containing `IncomingMessage` objects, each object per single received message. `ListIncomingMessagesResponse` object will contain also a `headers` property where you can find `X-Total-Results` (a total count of all received messages which are available in incoming box on your account), `X-Total-Pages` (a total number of all pages with results), `X-Current-Page` (A current page number) and `X-Limit` (messages count per single page) elements.

        :param page: Page number of results
        :param limit: Number of results on one page
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListIncomingMessagesRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/incoming",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListIncomingMessagesResponse(
                result=unmarshal_json_response(List[models.IncomingMessage], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListIncomingMessagesResponse:
        r"""List the received SMS messages

        Get the details of all received messages from your account incoming messages box. This method supports pagination so you have to pass `page` (number of page with received messages which you want to access) and a `limit` (max of received messages per page) named parameters. Messages are fetched from the latest one. This method will accept maximum **50** as `limit` parameter value.

        As a successful result a `ListIncomingMessagesResponse` object will be returned with `result` property of type `List[IncomingMessage]` containing `IncomingMessage` objects, each object per single received message. `ListIncomingMessagesResponse` object will contain also a `headers` property where you can find `X-Total-Results` (a total count of all received messages which are available in incoming box on your account), `X-Total-Pages` (a total number of all pages with results), `X-Current-Page` (A current page number) and `X-Limit` (messages count per single page) elements.

        :param page: Page number of results
        :param limit: Number of results on one page
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListIncomingMessagesRequest(
            page=page,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/incoming",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListIncomingMessagesResponse(
                result=unmarshal_json_response(List[models.IncomingMessage], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)

    def get_by_ids(
        self,
        *,
        ids: List[int],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetIncomingMessagesResponse:
        r"""Get the incoming messages by IDs

        Get the details of one or more received messages using their `ids`. This method accepts a `List[int]` as a `ids` named parameter containing unique incoming message *IDs*, which were given while receiving a messages. The method will accept maximum 50 identifiers in one call.

        As a successful result a `GetIncomingMessagesResponse` object will be returned with an `result` property of type `List[IncomingMessage]` containing `IncomingMessage` objects, each object per single received message. `GetIncomingMessagesResponse` object will contain also a `headers` property where you can find `X-Success-Count` (a count of incoming messages which were found and returned correctly) and `X-Error-Count` (count of incoming messages which were not found) elements.

        :param ids: List[str] with Message IDs assigned by the system. The system will accept a maximum of 50 identifiers in one call.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetIncomingMessagesRequest(
            ids=ids,
        )

        req = self._build_request(
            method="GET",
            path="/incoming/{ids}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetIncomingMessagesResponse(
                result=unmarshal_json_response(List[models.IncomingMessage], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_by_ids_async(
        self,
        *,
        ids: List[int],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetIncomingMessagesResponse:
        r"""Get the incoming messages by IDs

        Get the details of one or more received messages using their `ids`. This method accepts a `List[int]` as a `ids` named parameter containing unique incoming message *IDs*, which were given while receiving a messages. The method will accept maximum 50 identifiers in one call.

        As a successful result a `GetIncomingMessagesResponse` object will be returned with an `result` property of type `List[IncomingMessage]` containing `IncomingMessage` objects, each object per single received message. `GetIncomingMessagesResponse` object will contain also a `headers` property where you can find `X-Success-Count` (a count of incoming messages which were found and returned correctly) and `X-Error-Count` (count of incoming messages which were not found) elements.

        :param ids: List[str] with Message IDs assigned by the system. The system will accept a maximum of 50 identifiers in one call.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetIncomingMessagesRequest(
            ids=ids,
        )

        req = self._build_request_async(
            method="GET",
            path="/incoming/{ids}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.GetIncomingMessagesResponse(
                result=unmarshal_json_response(List[models.IncomingMessage], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)

    def remove_by_ids(
        self,
        *,
        ids: List[int],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RemoveIncomingMessagesResponse:
        r"""Remove the incoming messages from your inbox

        Remove incoming messages from your inbox using their `ids`. You have to pass the unique incoming message IDs as path parameter. If you want to remove multiple incoming messages at once, please separate their IDs with a comma. The system will accept maximum 50 identifiers in one call. If you need to remove larger volume of incoming messages, please split it to several separate requests.

        As a successful result an array with `RemovedIncomingMessage` objects will be returned, each object per single incoming message id. The `status` property will contain a status code of operation - `204` if incoming message was removed successfully and other code if an error occured with removing a given incoming message. In case of an error, an `error` property will contain `ErrorResponse` object with the details of an error.

        Response will also include meta-data headers: `X-Success-Count` (a count of incoming messages which were removed successfully), `X-Error-Count` (count of incoming messages which were not removed) and `X-Sandbox` (if a request was made in Sandbox or Production system).

        If you pass duplicated incoming message IDs in one call, API will process them only once. This request have to be authenticated using **API Access Token**.

        In case of an error, the `ErrorResponse` object will be returned with proper HTTP header status code (our error response complies with [RFC 9457](https://www.rfc-editor.org/rfc/rfc7807)).

        :param ids: List[str] with Message IDs assigned by the system. The system will accept a maximum of 50 identifiers in one call.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RemoveIncomingMessagesRequest(
            ids=ids,
        )

        req = self._build_request(
            method="DELETE",
            path="/incoming/{ids}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="removeIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.RemoveIncomingMessagesResponse(
                result=unmarshal_json_response(
                    List[models.RemovedIncomingMessage], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)

    async def remove_by_ids_async(
        self,
        *,
        ids: List[int],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RemoveIncomingMessagesResponse:
        r"""Remove the incoming messages from your inbox

        Remove incoming messages from your inbox using their `ids`. You have to pass the unique incoming message IDs as path parameter. If you want to remove multiple incoming messages at once, please separate their IDs with a comma. The system will accept maximum 50 identifiers in one call. If you need to remove larger volume of incoming messages, please split it to several separate requests.

        As a successful result an array with `RemovedIncomingMessage` objects will be returned, each object per single incoming message id. The `status` property will contain a status code of operation - `204` if incoming message was removed successfully and other code if an error occured with removing a given incoming message. In case of an error, an `error` property will contain `ErrorResponse` object with the details of an error.

        Response will also include meta-data headers: `X-Success-Count` (a count of incoming messages which were removed successfully), `X-Error-Count` (count of incoming messages which were not removed) and `X-Sandbox` (if a request was made in Sandbox or Production system).

        If you pass duplicated incoming message IDs in one call, API will process them only once. This request have to be authenticated using **API Access Token**.

        In case of an error, the `ErrorResponse` object will be returned with proper HTTP header status code (our error response complies with [RFC 9457](https://www.rfc-editor.org/rfc/rfc7807)).

        :param ids: List[str] with Message IDs assigned by the system. The system will accept a maximum of 50 identifiers in one call.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RemoveIncomingMessagesRequest(
            ids=ids,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/incoming/{ids}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="removeIncomingMessages",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.RemoveIncomingMessagesResponse(
                result=unmarshal_json_response(
                    List[models.RemovedIncomingMessage], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(
            http_res, ["400", "401", "403", "404", "4XX"], "application/problem+json"
        ):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)
        if utils.match_response(http_res, "5XX", "application/problem+json"):
            response_data = unmarshal_json_response(
                models.ErrorResponseErrorData, http_res
            )
            raise models.ErrorResponseError(response_data, http_res)

        raise models.SDKError("Unexpected response received", http_res)
